"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (_ref) {
  var t = _ref.types,
      template = _ref.template;

  var WEBPACK_PROP = "webpackRequireWeakId";
  var SERVER_PROP = "serverSideRequirePath";

  var webpackTemplate = template("() => require.resolveWeak(MODULE)");
  var serverTemplate = template("PATH.join(__dirname, MODULE)");

  return {
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        var _this = this;

        var opts = _extends({
          server: true,
          webpack: false
        }, this.opts);

        if (!opts.server && !opts.webpack) return;

        var source = path.node.source.value;
        if (source !== "react-loadable") return;

        var defaultSpecifier = path.get("specifiers").find(function (specifier) {
          return specifier.isImportDefaultSpecifier();
        });

        if (!defaultSpecifier) return;

        var bindingName = defaultSpecifier.node.local.name;
        var binding = path.scope.getBinding(bindingName);

        binding.referencePaths.forEach(function (refPath) {
          var callExpression = refPath.parentPath;
          if (!callExpression.isCallExpression()) return;

          var args = callExpression.get("arguments");
          if (args.length !== 1) throw callExpression.error;

          var options = args[0];
          if (!options.isObjectExpression()) return;

          var properties = options.get("properties");
          var propertiesMap = {};

          properties.forEach(function (property) {
            var key = property.get("key");
            propertiesMap[key.node.name] = property;
          });

          if ((!opts.webpack || properties[WEBPACK_PROP]) && (!opts.server || properties[SERVER_PROP])) {
            return;
          }

          var loaderMethod = propertiesMap.loader.get("value");
          var dynamicImport = void 0;

          loaderMethod.traverse({
            Import: function Import(path) {
              dynamicImport = path.parentPath;
              path.stop();
            }
          });

          if (!dynamicImport) return;

          var importedModule = dynamicImport.get("arguments")[0];

          if (opts.webpack && !propertiesMap[WEBPACK_PROP]) {
            var webpack = webpackTemplate({
              MODULE: importedModule.node
            }).expression;

            propertiesMap.loader.insertAfter(t.objectProperty(t.identifier(WEBPACK_PROP), webpack));
          }

          if (opts.server && !propertiesMap[SERVER_PROP]) {
            var server = serverTemplate({
              PATH: _this.addImport("path", "default", "path"),
              MODULE: importedModule.node
            }).expression;

            propertiesMap.loader.insertAfter(t.objectProperty(t.identifier(SERVER_PROP), server));
          }
        });
      }
    }
  };
};